<div class="popup {{ css_class }}" id="popup_base">
    <div class="centab">
        <div class="cencell">
            <div class="cencontent popup-wrapper">
                <div class="popup-content"></div>
                <div class="popup-controls">
                    <button class="close-popup button"><i class="fa-solid fa-light fa-times-circle"></i> Cancel</button>
                    <button class="confirm button"><i class="fa-solid fa-light fa-check-circle"></i> Confirm</button>
                </div>
            </div>
        </div>
    </div>
</div>