{% extends 'content_blocks/partials/fields/previews/base.html' %}

{% block main %}
    {% if field.value %}
        <img class="image-preview" src="{{ field.value.url }}" />
    {% else %}
        <i class="fa-solid fa-thin fa-image-slash fa-2x no-image-preview"></i>
    {% endif %}
{% endblock %}