<div class="hidden" id="reset_popup_content">
    <h2>Reset Content Blocks</h2>

    <p>These content blocks will be reset to their published state. This action cannot be undone. Are you sure you want to continue?</p>
</div>